/*

  /-------------------------------------------------\
  |                                                 |
  |                                                 |
  |     /=====\     /=====\     |\          /|      |
  |    /       \   /       \    | \        / |      |
  |    |           |            |  \      /  |      |
  |    |           |            |   \    /   |      |
  |    \       /   \       /    |    \  /    |      |
  |     \=====/     \=====/     |     \/     |      |
  |                                                 |
  |                                                 |
  \-------------------------------------------------/

  ===================================================

  Name:       CCM (collect.C)
  
  Written by: Damian J. J. Farnell

  Date:       Wed Mar 15 15:20:28 GMT 2000

  Updated:    Joerg Schulenburg 2004

  ===================================================

  ---------------------------------------------------

  PROGRAM NAME: collect.C 

  PURPOSE: collect same configurations to reduce loops

  ---------------------------------------------------

*/

#include<iostream.h>
#include<stdlib.h>
#include<stdlib.h>
#include<math.h>
#include<stdio.h>
#include<string.h>

#include "../include/Constants.h"
#include "../include/Externals.h"
#include "../include/FunctionPrototypes.h"
#include "../include/nr.h"
#include "../include/nrutil.h"

#if (PARALLEL_FLAG>0)
#include<mpi.h>
#endif

#define DO_COLLECT 1  /* set to 0 to switch off collect, default = 1 */

int descriptors[MAX_NUMBER_TERMS_IN_CCM_EQNS+1][6];  /* local = stack? */
double multipliers[MAX_NUMBER_TERMS_IN_CCM_EQNS+1];
int counter=0;

#if DO_COLLECT
void collect_term(int indices[6],double dummy, FILE *fp1) 
{

  int j, config_index, term_index;
  static int n_out=0; /* count outputs, avoid GB of debugging output */

  int found;

  config_index=100000;
  term_index=100000;
  if (counter){
    config_index=descriptors[counter][0];
    term_index=  descriptors[counter][1];
  }

      if (Control.output > 4 && myid == 0)
	printf("IN: Config=%d : Term = %d : CCM = (%d,%d,%d,%d) : mult=%10.7f\n",
	       indices[0],indices[1],
	       indices[2],indices[3],indices[4],indices[5],dummy); 
      
      if (config_index < indices[0]
         || term_index < indices[1]
         ||          0 > indices[0]
         || NumberConfigurations < indices[0]) {
	
	// first save information 
	for(j=1;j<=counter;j++){
	  
	  fwrite(  descriptors[j] ,sizeof(int),6,fp1);
	  fwrite(&(multipliers[j]),sizeof(double),1,fp1);
	  
	  // printf out information
	  
	  if (Control.output > 1 && myid == 0 && n_out < 1000) {
	    printf("OUT: Config=%d : Term = %d : CCM = (%d,%d,%d,%d) : "
		   "mult=%10.7f\n",
		   descriptors[j][0],descriptors[j][1],
		   descriptors[j][2],descriptors[j][3],
                   descriptors[j][4],descriptors[j][5],
		   multipliers[j]);
            if (n_out+1==1000)
              printf("excessive output is stopped after 1000 terms\n");
          }
          n_out++;
	  
	  // now overwrite the old stored information 
	  
	}
	counter=0;
      }
      
      found=0;
      
      for(j=1;j<=counter;j++) {
	if (indices[0]==descriptors[j][0] &&
	    indices[1]==descriptors[j][1]) {
	  
	  if ((indices[2]==descriptors[j][2] &&
	       indices[3]==descriptors[j][3] &&
	       indices[4]==descriptors[j][4] &&
	       indices[5]==descriptors[j][5]) ||
	      (indices[2]==descriptors[j][3] &&
	       indices[3]==descriptors[j][2] &&
	       indices[4]==descriptors[j][4] &&
	       indices[5]==descriptors[j][5]) ||
	      (indices[2]==descriptors[j][4] &&
	       indices[3]==descriptors[j][3] &&
	       indices[4]==descriptors[j][2] &&
	       indices[5]==descriptors[j][5]) ||
	      (indices[2]==descriptors[j][5] &&
	       indices[3]==descriptors[j][3] &&
	       indices[4]==descriptors[j][4] &&
	       indices[5]==descriptors[j][2]) ||
	      (indices[2]==descriptors[j][2] &&
	       indices[3]==descriptors[j][4] &&
	       indices[4]==descriptors[j][3] &&
	       indices[5]==descriptors[j][5]) ||
	      (indices[2]==descriptors[j][2] &&
	       indices[3]==descriptors[j][5] &&
	       indices[4]==descriptors[j][4] &&
	       indices[5]==descriptors[j][3]) ||
	      (indices[2]==descriptors[j][2] &&
	       indices[3]==descriptors[j][3] &&
	       indices[4]==descriptors[j][5] &&
	       indices[5]==descriptors[j][4])){
	    multipliers[j]+=dummy;
	    found=1;
	    break;
	  }
	}
      }
      
      if (indices[0]<=NumberConfigurations && indices[0]>=0)
      if ((!found) && (!(indices[0] == 0 && 
			 indices[1] == 0 &&
			 indices[2] == 0 &&
			 indices[3] == 0 &&
			 indices[4] == 0 &&
			 indices[5] == 0))) {
	counter++;  /* potential overflow! */
        if (counter > MAX_NUMBER_TERMS_IN_CCM_EQNS) {
           printf("number_terms > MAX_NUMBER_TERMS_IN_CCM_EQNS, exit!");
           exit(1);
        }
	descriptors[counter][0]=indices[0];
	descriptors[counter][1]=indices[1];
	descriptors[counter][2]=indices[2];
	descriptors[counter][3]=indices[3];
	descriptors[counter][4]=indices[4];
	descriptors[counter][5]=indices[5];
	multipliers[counter]=dummy;
      }
    
      if (indices[0]>NumberConfigurations || indices[0]<0) {
        fwrite(indices,sizeof(int),6,fp1);
        fwrite(&dummy,sizeof(double),1,fp1);
      }
   
}
#else /* dont collect */
void collect_term(int indices[6],double dummy, FILE *fp1) 
{
  static int n_out=0;

  fwrite(indices,sizeof(int),6,fp1);
  fwrite( &dummy,sizeof(double),1,fp1);
	  
  if (Control.output > 1 && myid == 0 && n_out < 1000) {
     printf("OUT: Config=%d : Term = %d : CCM = (%d,%d,%d,%d) : "
     "mult=%10.7f\n",
     indices[0],indices[1],
     indices[2],indices[3],
     indices[4],indices[5],
     dummy);
     if (n_out+1==1000)
        printf("excessive output is stopped after 1000 terms\n");
  }
  n_out++;
}
#endif

void collect(void) 
{
  /* obsolete */
}

